@@ -3,7 +3,7 @@ mod relocation;
 
 use object::{
     read::{Object as ElfObject, ObjectSection, Section as ObjSection},
-    Endianness, ObjectSymbol, ObjectSymbolTable, RelocationTarget, SectionIndex,
+    Endianness, ObjectSymbol, ObjectSymbolTable, RelocationTarget, SectionIndex, SymbolKind,
 };
 use std::{
     collections::HashMap,
@@ -84,6 +84,7 @@ pub enum ProgramSection {
     CgroupSkbIngress { name: String },
     CgroupSkbEgress { name: String },
     LircMode2 { name: String },
+    PerfEvent { name: String },
 }
 
 impl ProgramSection {
@@ -104,6 +105,7 @@ impl ProgramSection {
             ProgramSection::CgroupSkbIngress { name } => name,
             ProgramSection::CgroupSkbEgress { name } => name,
             ProgramSection::LircMode2 { name } => name,
+            ProgramSection::PerfEvent { name } => name,
         }
     }
 }
@@ -144,6 +146,7 @@ impl FromStr for ProgramSection {
             "cgroup_skb/ingress" => CgroupSkbIngress { name },
             "cgroup_skb/egress" => CgroupSkbEgress { name },
             "lirc_mode2" => LircMode2 { name },
+            "perf_event" => PerfEvent { name },
             _ => {
                 return Err(ParseError::InvalidProgramSection {
                     section: section.to_owned(),
@@ -181,6 +184,7 @@ impl Object {
                     address: symbol.address(),
                     size: symbol.size(),
                     is_definition: symbol.is_definition(),
+                    is_text: symbol.kind() == SymbolKind::Text,
                 };
                 bpf_obj
                     .symbols_by_index
@@ -242,7 +246,7 @@ impl Object {
         let mut symbols_by_address = HashMap::new();
 
         for sym in self.symbols_by_index.values() {
-            if sym.is_definition && sym.section_index == Some(section.index) {
+            if sym.is_definition && sym.is_text && sym.section_index == Some(section.index) {
                 if symbols_by_address.contains_key(&sym.address) {
                     return Err(ParseError::SymbolTableConflict {
                         section_index: section.index.0,
@@ -381,10 +385,10 @@ pub enum ParseError {
     #[error("error parsing map `{name}`")]
     InvalidMapDefinition { name: String },
 
-    #[error("two or more symbols in section `{section_index}` have the same address {address:x}")]
+    #[error("two or more symbols in section `{section_index}` have the same address {address:#X}")]
     SymbolTableConflict { section_index: usize, address: u64 },
 
-    #[error("unknown symbol in section `{section_index}` at address {address:x}")]
+    #[error("unknown symbol in section `{section_index}` at address {address:#X}")]
     UnknownSymbol { section_index: usize, address: u64 },
 
     #[error("invalid symbol, index `{index}` name: {}", .name.as_ref().unwrap_or(&"[unknown]".into()))]
@@ -555,6 +559,7 @@ fn is_program_section(name: &str) -> bool {
         "kprobe",
         "kretprobe",
         "lirc_mode2",
+        "perf_event",
         "sk_msg",
         "sk_skb/stream_parser",
         "sk_skb/stream_verdict",
