@@ -20,8 +20,8 @@ enum RelocationError {
     #[error("unknown symbol, index `{index}`")]
     UnknownSymbol { index: usize },
 
-    #[error("section `{section_index}` not found, referenced by symbol `{}`",
-            .symbol_name.clone().unwrap_or_else(|| .symbol_index.to_string()))]
+    #[error("section `{section_index}` not found, referenced by symbol `{}` #{symbol_index}",
+            .symbol_name.clone().unwrap_or_else(|| "".to_string()))]
     SectionNotFound {
         section_index: usize,
         symbol_index: usize,
@@ -57,6 +57,7 @@ pub(crate) struct Symbol {
     pub(crate) address: u64,
     pub(crate) size: u64,
     pub(crate) is_definition: bool,
+    pub(crate) is_text: bool,
 }
 
 impl Object {
@@ -256,19 +257,20 @@ impl<'a> FunctionLinker<'a> {
                 continue;
             }
 
-            let callee_address = if let Some(rel) = rel_info((ins_index * INS_SIZE) as u64) {
-                // We have a relocation entry for the instruction at `ins_index`, the address of
-                // the callee is the address of the relocation's target symbol.
-                rel_target_address(rel, self.symbol_table)?
-            } else {
-                // The caller and the callee are in the same ELF section and this is a pc-relative
-                // call. Resolve the pc-relative imm to an absolute address.
-                let ins_size = INS_SIZE as i64;
-                (fun.section_offset as i64
-                    + ((ins_index - start_ins) as i64) * ins_size
-                    + (program.instructions[ins_index].imm + 1) as i64 * ins_size)
-                    as u64
-            };
+            let callee_address =
+                if let Some(rel) = rel_info(((ins_index - start_ins) * INS_SIZE) as u64) {
+                    // We have a relocation entry for the instruction at `ins_index`, the address of
+                    // the callee is the address of the relocation's target symbol.
+                    rel_target_address(rel, self.symbol_table)?
+                } else {
+                    // The caller and the callee are in the same ELF section and this is a pc-relative
+                    // call. Resolve the pc-relative imm to an absolute address.
+                    let ins_size = INS_SIZE as i64;
+                    (fun.section_offset as i64
+                        + ((ins_index - start_ins) as i64) * ins_size
+                        + (program.instructions[ins_index].imm + 1) as i64 * ins_size)
+                        as u64
+                };
 
             // lookup and link the callee if it hasn't been linked already. `callee_ins_index` will
             // contain the instruction index of the callee inside the program.
